@@ -47,6 +47,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <time.h>
 #include <unistd.h>
 
 #include "error.h"
@@ -55,21 +57,20 @@
 #include "xalloc.h"
 
 #include "dhcpd-pools.h"
-#include "defaults.h"
 
-char *(*cidr_last)(union ipaddr_t *restrict addr, const int mask);
+static char *(*cidr_last) (union ipaddr_t *restrict addr, const int mask);
 static char *cidr_last_v4(union ipaddr_t *restrict addr, const int mask);
 static char *cidr_last_v6(union ipaddr_t *restrict addr, const int mask);
 
 /*! \brief Set function pointers depending on IP version.
  * \param ip IP version.
  */
-void set_ipv_functions(int version)
+void set_ipv_functions(struct conf_t *state, int version)
 {
 	switch (version) {
 
 	case IPv4:
-		config.ip_version = version;
+		state->ip_version = version;
 		add_lease = add_lease_v4;
 		copy_ipaddr = copy_ipaddr_v4;
 		find_lease = find_lease_v4;
@@ -83,7 +84,7 @@ void set_ipv_functions(int version)
 		break;
 
 	case IPv6:
-		config.ip_version = version;
+		state->ip_version = version;
 		add_lease = add_lease_v6;
 		copy_ipaddr = copy_ipaddr_v6;
 		find_lease = find_lease_v6;
@@ -97,7 +98,7 @@ void set_ipv_functions(int version)
 		break;
 
 	case IPvUNKNOWN:
-		config.ip_version = version;
+		state->ip_version = version;
 		add_lease = add_lease_init;
 		copy_ipaddr = copy_ipaddr_init;
 		find_lease = find_lease_init;
@@ -122,21 +123,23 @@ void set_ipv_functions(int version)
  * \param dst An union which will hold conversion result.
  * \return Was parsing successful.
  */
-int parse_ipaddr_init(const char *restrict src, union ipaddr_t *restrict dst)
+int parse_ipaddr_init(struct conf_t *state, const char *restrict src, union ipaddr_t *restrict dst)
 {
 	struct in_addr addr;
 	struct in6_addr addr6;
 
 	if (inet_aton(src, &addr) == 1)
-		set_ipv_functions(IPv4);
+		set_ipv_functions(state, IPv4);
 	else if (inet_pton(AF_INET6, src, &addr6) == 1)
-		set_ipv_functions(IPv6);
+		set_ipv_functions(state, IPv6);
 	else
 		return 0;
-	return parse_ipaddr(src, dst);
+	return parse_ipaddr(state, src, dst);
 }
 
-int parse_ipaddr_v4(const char *restrict src, union ipaddr_t *restrict dst)
+int parse_ipaddr_v4(struct conf_t *state
+		    __attribute__ ((unused)), const char *restrict src,
+		    union ipaddr_t *restrict dst)
 {
 	int rv;
 	struct in_addr addr;
@@ -146,7 +149,9 @@ int parse_ipaddr_v4(const char *restrict src, union ipaddr_t *restrict dst)
 	return rv == 1;
 }
 
-int parse_ipaddr_v6(const char *restrict src, union ipaddr_t *restrict dst)
+int parse_ipaddr_v6(struct conf_t *state
+		    __attribute__ ((unused)), const char *restrict src,
+		    union ipaddr_t *restrict dst)
 {
 	int rv;
 	struct in6_addr addr;
@@ -156,6 +161,10 @@ int parse_ipaddr_v6(const char *restrict src, union ipaddr_t *restrict dst)
 	return rv == 1;
 }
 
+/*! \brief Convert string to a desimal format network marks.
+ * \param src Digit that should be a network mask.
+ * \return Network mask, or -1 when failing.
+ */
 static int strtol_mask(const char *str)
 {
 	long num;
@@ -175,6 +184,10 @@ static int strtol_mask(const char *str)
 	return -1;
 }
 
+/*! \brief Find last address in IPv4 range by using cidr format.
+ * \param addr Pointer to memory where address needs to be stored.
+ * \return Allocated string format of the address.
+ */
 static char *cidr_last_v4(union ipaddr_t *restrict addr, const int mask)
 {
 	union ipaddr_t last_ip;
@@ -185,12 +198,16 @@ static char *cidr_last_v4(union ipaddr_t *restrict addr, const int mask)
 		netmask = (1U << (32 - mask)) - 1;
 	else
 		netmask = 0;
-	last_ip.v4 =  addr->v4 | netmask;
+	last_ip.v4 = addr->v4 | netmask;
 
 	ip = ntop_ipaddr(&last_ip);
 	return xstrdup(ip);
 }
 
+/*! \brief Find last address in IPv6 range by using cidr format.
+ * \param addr Pointer to memory where address needs to be stored.
+ * \return Allocated string format of the address.
+ */
 static char *cidr_last_v6(union ipaddr_t *restrict addr, const int mask)
 {
 	union ipaddr_t bitmask;
@@ -210,7 +227,11 @@ static char *cidr_last_v6(union ipaddr_t *restrict addr, const int mask)
 	return xstrdup(ip);
 }
 
-void parse_cidr(struct range_t *range_p, const char *word)
+/*! \brief Convert a cidr notated address to a range.
+ * \param range_p Pointer to memory where addresses need to be stored.
+ * \param word A range as a cidr string.
+ */
+void parse_cidr(struct conf_t *state, struct range_t *range_p, const char *word)
 {
 	char *divider;
 	int mask;
@@ -222,22 +243,21 @@ void parse_cidr(struct range_t *range_p, const char *word)
 	*divider++ = '\0';
 	mask = strtol_mask(divider);
 	if (mask < 0)
-		error(EXIT_FAILURE, 0, "cidr %s invalid mask %s", word,
-		      divider);
-	if (config.ip_version == IPvUNKNOWN) {
+		error(EXIT_FAILURE, 0, "cidr %s invalid mask %s", word, divider);
+	if (state->ip_version == IPvUNKNOWN) {
 		if (!strchr(word, ':'))
-			set_ipv_functions(IPv4);
+			set_ipv_functions(state, IPv4);
 		else
-			set_ipv_functions(IPv6);
+			set_ipv_functions(state, IPv6);
 	}
 
 	/* start of the range is easy */
-	parse_ipaddr(word, &addr);
+	parse_ipaddr(state, word, &addr);
 	copy_ipaddr(&range_p->first_ip, &addr);
 
 	/* end of the range depends cidr size */
 	last = cidr_last(&addr, mask);
-	parse_ipaddr(last, &addr);
+	parse_ipaddr(state, last, &addr);
 	copy_ipaddr(&range_p->last_ip, &addr);
 	free(last);
 }
@@ -278,7 +298,7 @@ const char *ntop_ipaddr_init(const union ipaddr_t *ip __attribute__ ((unused)))
 
 const char *ntop_ipaddr_v4(const union ipaddr_t *ip)
 {
-	static char buffer[sizeof("255.255.255.255")];
+	static char buffer[INET_ADDRSTRLEN];
 	struct in_addr addr;
 
 	addr.s_addr = htonl(ip->v4);
@@ -287,7 +307,7 @@ const char *ntop_ipaddr_v4(const union ipaddr_t *ip)
 
 const char *ntop_ipaddr_v6(const union ipaddr_t *ip)
 {
-	static char buffer[sizeof("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255")];
+	static char buffer[INET6_ADDRSTRLEN];
 	struct in6_addr addr;
 
 	memcpy(addr.s6_addr, ip->v6, sizeof(addr.s6_addr));
@@ -332,17 +352,14 @@ double get_range_size_v6(const struct range_t *r)
  * \param str A line from dhcpd.conf
  * \return prefix_t enum value
  */
-int
-#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
-    __attribute__ ((hot))
-#endif
-    xstrstr_init(const char *restrict str)
+int xstrstr_init(struct conf_t *state, const char *restrict str)
 {
-	if (memcmp("lease ", str, 6)) {
-		set_ipv_functions(IPv4);
+	if (!memcmp("lease ", str, 6)) {
+		set_ipv_functions(state, IPv4);
 		return PREFIX_LEASE;
-	} else if (memcmp("  iaaddr ", str, 9)) {
-		set_ipv_functions(IPv6);
+	}
+	if (!memcmp("  iaaddr ", str, 9)) {
+		set_ipv_functions(state, IPv6);
 		return PREFIX_LEASE;
 	}
 	return NUM_OF_PREFIX;
@@ -358,7 +375,7 @@ int
 #if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
     __attribute__ ((hot))
 #endif
-    xstrstr_v4(const char *restrict str)
+    xstrstr_v4(struct conf_t *state __attribute__ ((unused)), const char *restrict str)
 {
 	size_t len;
 
@@ -394,6 +411,8 @@ int
 			if (!memcmp("  hardware ethernet", str, 19))
 				return PREFIX_HARDWARE_ETHERNET;
 			break;
+		default:
+			break;
 		}
 	}
 	if (!memcmp("lease ", str, 6))
@@ -411,7 +430,7 @@ int
 #if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
     __attribute__ ((hot))
 #endif
-    xstrstr_v6(const char *restrict str)
+    xstrstr_v6(struct conf_t *state __attribute__ ((unused)), const char *restrict str)
 {
 	size_t len;
 
@@ -447,6 +466,8 @@ int
 			if (!memcmp("  hardware ethernet", str, 19))
 				return PREFIX_HARDWARE_ETHERNET;
 			break;
+		default:
+			break;
 		}
 	}
 	if (!memcmp("  iaaddr ", str, 9))
@@ -454,6 +475,22 @@ int
 	return NUM_OF_PREFIX;
 }
 
+/*! \brief Parse option argument color mode.
+ *
+ * \param Color mode string.
+ * \return color mode enum.
+ */
+int parse_color_mode(const char *restrict arg)
+{
+	if (!strcmp(arg, "always"))
+		return color_on;
+	if (!strcmp(arg, "auto"))
+		return color_auto;
+	if (!strcmp(arg, "never"))
+		return color_off;
+	return color_unknown;
+}
+
 /*! \brief Return a double floating point value.
  *
  * \param str String to be converted to a double.
@@ -478,56 +515,79 @@ double strtod_or_err(const char *restrict str, const char *restrict errmesg)
 }
 
 /*! \brief Reverse range.
- * Used before output, if a caller has requested reverse sorting.
- * FIXME: The temporary memory area handling should be internal to this
- * function, not a parameter.
- *
- * \param flip_me The range that needs to be inverted.
- * \param tmp_ranges Temporary memory area for the flip. */
-void flip_ranges(struct range_t *restrict flip_me, struct range_t *restrict tmp_ranges)
+ * Used before output, if a caller has requested reverse sorting. */
+void flip_ranges(struct conf_t *state)
 {
-	unsigned int i = num_ranges - 1, j;
+	unsigned int i = state->num_ranges - 1, j;
+	struct range_t *tmp_ranges;
 
-	for (j = 0; j < num_ranges; j++, i--)
-		*(tmp_ranges + j) = *(flip_me + i);
-	memcpy(flip_me, tmp_ranges, num_ranges * sizeof(struct range_t));
+	tmp_ranges = xmalloc(sizeof(struct range_t) * state->num_ranges);
+	for (j = 0; j < state->num_ranges; j++, i--)
+		*(tmp_ranges + j) = *(state->ranges + i);
+	memcpy(state->ranges, tmp_ranges, state->num_ranges * sizeof(struct range_t));
+	free(tmp_ranges);
 }
 
 /*! \brief Free memory, flush buffers etc. */
-void clean_up(void)
+void clean_up(struct conf_t *state)
 {
 	struct output_sort *cur, *next;
+	struct shared_network_t *c, *n;
 
 	/* Just in case there something in buffers */
 	if (fflush(NULL))
 		error(EXIT_FAILURE, errno, "clean_up: fflush");
-	free(config.dhcpdconf_file);
-	free(config.dhcpdlease_file);
-	free(config.output_file);
-	free(ranges);
-	delete_all_leases();
-	if (shared_networks) {
-		unsigned int i;
-
-		num_shared_networks++;
-		for (i = 0; i < num_shared_networks; i++)
-			free((shared_networks + i)->name);
-		free(shared_networks);
-	}
-	for (cur = config.sorts; cur; cur = next) {
+	free(state->ranges);
+	delete_all_leases(state);
+	for (cur = state->sorts; cur; cur = next) {
 		next = cur->next;
 		free(cur);
 	}
+	for (c = state->shared_net_root; c; c = n) {
+		n = c->next;
+		free(c->name);
+		free(c);
+	}
+}
+
+/*! \brief Print a time stamp of a path or now to output file. */
+void dp_time_tool(FILE *file, const char *path, const int epoch)
+{
+	struct stat st;
+
+	if (path)
+		stat(path, &st);
+	else
+		clock_gettime(CLOCK_REALTIME, &st.st_mtim);
+	if (epoch)
+		fprintf(file, "%ld", st.st_mtim.tv_sec);
+	else {
+		char time_stamp[64];
+		const time_t mtime = st.st_mtim.tv_sec;
+		struct tm tm;
+
+		localtime_r(&mtime, &tm);
+		strftime(time_stamp, sizeof(time_stamp), "%FT%T%z", &tm);
+		fprintf(file, "%s", time_stamp);
+	}
 }
 
 /*! \brief A version printing. */
 void __attribute__ ((__noreturn__)) print_version(void)
 {
+#define stringify(s) #s
+#define stringify_value(s) stringify(s)
 	fprintf(stdout, "%s\n"
-		"Original design by Sami Kerola.\n"
+		"Original design and maintainer Sami Kerola.\n"
+		"uthash %s by Troy D. Hanson.\n"
 		"XML support by Dominic Germain, Sogetel inc.\n"
-		"IPv6 support by Cheer Xiao.\n\n"
-		"The software has FreeBSD License.\n", PACKAGE_STRING);
+		"IPv6 support by Cheer Xiao.\n"
+		"Mustach templating support by Jose Bollo.\n"
+		"  The dhcpd-pools is FreeBSD Licensed,\n"
+		"  uthash uses BSD license,\n"
+		"  gnulib parts are mostly GPL,\n"
+		"  and mustache uses Apache License.\n", PACKAGE_STRING,
+		stringify_value(UTHASH_VERSION));
 	exit(EXIT_SUCCESS);
 }
 
@@ -550,6 +610,9 @@ void __attribute__ ((__noreturn__)) usage(int status)
 	fputs(		"                           j for json\n", out);
 	fputs(		"                           J for json with active lease details\n", out);
 	fputs(		"                           c for comma separated values\n", out);
+#ifdef BUILD_MUSTACH
+	fputs(		"      --mustach=FILE     output using mustach template file\n", out);
+#endif
 	fputs(		"  -s, --sort=[nimcptTe]  sort ranges by\n", out);
 	fputs(		"                           n name\n", out);
 	fputs(		"                           i IP\n", out);
@@ -562,8 +625,11 @@ void __attribute__ ((__noreturn__)) usage(int status)
 	fputs(		"  -r, --reverse          reverse order sort\n", out);
 	fputs(		"  -o, --output=FILE      output into a file\n", out);
 	fputs(		"  -L, --limit=NR         output limit mask 77 - 00\n", out);
-	fputs(		"      --warning=PERC     set warning alarming limit\n", out);
-	fputs(		"      --critical=PERC    set critical alarming limit\n", out);
+	fputs(		"      --color=WHEN       use colors 'always', 'never', or 'auto'\n", out);
+	fputs(		"      --warning=PERC     set warning alarming threshold\n", out);
+	fputs(		"      --critical=PERC    set critical alarming threshold\n", out);
+	fputs(		"      --skip=WHAT        do not print threshold 'ok', 'warning', 'critical',\n", out);
+	fputs(		"                           'minsize', or 'suppressed'\n", out);
 	fputs(		"      --warn-count=NR    a number of free leases before warning raised\n", out);
 	fputs(		"      --crit-count=NR    a number of free leases before critical raised\n", out);
 	fputs(		"      --minsize=size     disable alarms for small ranges and shared-nets\n", out);
