@@ -40,6 +40,7 @@
 
 #include <config.h>
 
+#include <math.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,6 +48,7 @@
 #include "error.h"
 #include "progname.h"
 #include "quote.h"
+#include "xalloc.h"
 
 #include "dhcpd-pools.h"
 
@@ -117,9 +119,14 @@ int rangecomp(const void *restrict r1, const void *restrict r2)
  */
 int comp_double(double f1, double f2)
 {
-	return f1 < f2 ? -1 : f1 > f2 ? 1 : 0;
+        if (isless(f1, f2))
+                return -1;
+        else if (isless(f2, f1))
+                return 1;
+        return 0;
 }
 
+
 /*! \brief Compare two range_t by their first_ip.
  * \param r1,r2 Pointers to data to compare.
  * \return Like strcmp.
@@ -239,8 +246,8 @@ comparer_t field_selector(char c)
 	default:
 		{
 			char str[2] = { c, '\0' };
-			clean_up();
-			error(EXIT_FAILURE, 0, "field_selector: unknown sort order: %s", quote(str));
+			error(EXIT_FAILURE, 0, "field_selector: unknown sort order: %s",
+			      quote(str));
 		}
 	}
 	return NULL;
@@ -251,12 +258,12 @@ comparer_t field_selector(char c)
  * \param right The right side of the merge sort.
  * \return Relevant for merge sort decision.
  */
-static int merge(struct range_t *restrict left, struct range_t *restrict right)
+static int merge(struct conf_t *state, struct range_t *restrict left, struct range_t *restrict right)
 {
 	struct output_sort *p;
 	int ret;
 
-	for (p = config.sorts; p; p = p->next) {
+	for (p = state->sorts; p; p = p->next) {
 		if (p->func == NULL) {
 			/* String sorting is special. */
 			ret = strcmp(left->shared_net->name, right->shared_net->name);
@@ -274,43 +281,46 @@ static int merge(struct range_t *restrict left, struct range_t *restrict right)
 }
 
 /*! \brief Mergesort for range table.
- * \param orig Pointer to range that is requested to be sorted.
- * \param size Number of ranges to be sorted.
- * \param temp Temporary memory space, needed when a values has to be
- * flipped.
  */
-void mergesort_ranges(struct range_t *restrict orig, int size, struct range_t *restrict temp)
+void mergesort_ranges(struct conf_t *state, struct range_t *restrict orig, unsigned int size,
+		      struct range_t *restrict temp, const int root_call)
 {
-	int left, right, i;
+	unsigned int left, i, u_right;
 	struct range_t hold;
-
 	/* Merge sort split size */
-	static const int MIN_MERGE_SIZE = 8;
+	static const unsigned int MIN_MERGE_SIZE = 8;
+
+	if (temp == NULL)
+		temp = xmalloc(sizeof(struct range_t) * size);
 
 	if (size < MIN_MERGE_SIZE) {
 		for (left = 0; left < size; left++) {
+			int s_right;
+
 			hold = *(orig + left);
-			for (right = left - 1; 0 <= right; right--) {
-				if (merge((orig + right), &hold))
+			for (s_right = left - 1; 0 <= s_right; s_right--) {
+				if (merge(state, (orig + s_right), &hold))
 					break;
-				*(orig + right + 1) = *(orig + right);
+				*(orig + s_right + 1) = *(orig + s_right);
 			}
-			*(orig + right + 1) = hold;
+			*(orig + s_right + 1) = hold;
 		}
+		if (root_call)
+			free(temp);
 		return;
 	}
-	mergesort_ranges(orig, size / 2, temp);
-	mergesort_ranges(orig + size / 2, size - size / 2, temp);
+	mergesort_ranges(state, orig, size / 2, temp, 0);
+	mergesort_ranges(state, orig + size / 2, size - size / 2, temp, 0);
 	left = 0;
-	right = size / 2;
+	u_right = size / 2;
 	i = 0;
-	while (left < size / 2 && right < size) {
-		if (merge((orig + left), (orig + right))) {
+	while (left < size / 2 && u_right < size) {
+		if (merge(state, (orig + left), (orig + u_right))) {
 			*(temp + i) = *(orig + left);
 			left++;
 		} else {
-			*(temp + i) = *(orig + right);
-			right++;
+			*(temp + i) = *(orig + u_right);
+			u_right++;
 		}
 		i++;
 	}
@@ -319,10 +329,13 @@ void mergesort_ranges(struct range_t *restrict orig, int size, struct range_t *r
 		left++;
 		i++;
 	}
-	while (right < size) {
-		*(temp + i) = *(orig + right);
-		right++;
+	while (u_right < size) {
+		*(temp + i) = *(orig + u_right);
+		u_right++;
 		i++;
 	}
 	memcpy(orig, temp, size * sizeof(struct range_t));
+
+	if (root_call)
+		free(temp);
 }
